# Generated by Django 5.0.6 on 2024-08-13 09:07

import django.core.validators
import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("dcim", "0187_alter_device_vc_position"),
        ("extras", "0115_convert_dashboard_widgets"),
        ("ipam", "0069_gfk_indexes"),
        ("slurpit_netbox", "0016_slurpitprefix_ignore_description_and_more"),
        ("tenancy", "0015_contactassignment_rename_content_type"),
    ]

    operations = [
        migrations.CreateModel(
            name="SlurpitVLAN",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False
                    ),
                ),
                ("created", models.DateTimeField(auto_now_add=True, null=True)),
                ("last_updated", models.DateTimeField(auto_now=True, null=True)),
                (
                    "custom_field_data",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        encoder=utilities.json.CustomFieldJSONEncoder,
                    ),
                ),
                ("description", models.CharField(blank=True, max_length=200)),
                ("comments", models.TextField(blank=True)),
                (
                    "vid",
                    models.PositiveSmallIntegerField(
                        validators=[
                            django.core.validators.MinValueValidator(1),
                            django.core.validators.MaxValueValidator(4094),
                        ]
                    ),
                ),
                ("name", models.CharField(max_length=64)),
                ("status", models.CharField(default="active", max_length=50)),
                ("enable_reconcile", models.BooleanField(default=False)),
                (
                    "group",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="slurpit_vlans",
                        to="ipam.vlangroup",
                    ),
                ),
                (
                    "role",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="slurpit_vlans",
                        to="ipam.role",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="slurpit_vlans",
                        to="dcim.site",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        through="extras.TaggedItem", to="extras.Tag"
                    ),
                ),
                (
                    "tenant",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="slurpit_vlans",
                        to="tenancy.tenant",
                    ),
                ),
            ],
            options={
                "verbose_name": "VLAN",
                "verbose_name_plural": "VLANs",
                "ordering": ("site", "group", "vid", "pk"),
            },
        ),
        migrations.AddConstraint(
            model_name="slurpitvlan",
            constraint=models.UniqueConstraint(
                fields=("group", "vid"),
                name="slurpit_netbox_slurpitvlan_unique_group_vid",
            ),
        ),
        migrations.AddConstraint(
            model_name="slurpitvlan",
            constraint=models.UniqueConstraint(
                fields=("group", "name"),
                name="slurpit_netbox_slurpitvlan_unique_group_name",
            ),
        ),
    ]



import django.db.models.deletion
from django.db import migrations, models


def populate_denormalized_fields(apps, schema_editor):
    """
    Copy site ForeignKey values to the scope GFK.
    """
    SlurpitPrefix = apps.get_model('slurpit_netbox', 'SlurpitPrefix')

    prefixes = SlurpitPrefix.objects.filter(site__isnull=False).prefetch_related('site')
    for prefix in prefixes:
        prefix._region_id = prefix.site.region_id
        prefix._site_group_id = prefix.site.group_id
        prefix._site_id = prefix.site_id

    SlurpitPrefix.objects.bulk_update(prefixes, ['_region', '_site_group', '_site'])


class Migration(migrations.Migration):
    dependencies = [
        ('slurpit_netbox', '0024_alter_slurpitprefix_scope'),
    ]

    operations = [
        migrations.AddField(
            model_name='slurpitprefix',
            name='_location',
            field=models.ForeignKey(
                blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='dcim.location'
            ),
        ),
        migrations.AddField(
            model_name='slurpitprefix',
            name='_region',
            field=models.ForeignKey(
                blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='dcim.region'
            ),
        ),
        migrations.AddField(
            model_name='slurpitprefix',
            name='_site',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='dcim.site'),
        ),
        migrations.AddField(
            model_name='slurpitprefix',
            name='_site_group',
            field=models.ForeignKey(
                blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='dcim.sitegroup'
            ),
        ),
        # Populate denormalized FK values
        migrations.RunPython(code=populate_denormalized_fields, reverse_code=migrations.RunPython.noop),
        # Delete the site ForeignKey
        migrations.RemoveField(
            model_name='slurpitprefix',
            name='site',
        ),
    ]



from .onboarding import SlurpitImportedDeviceListView, SlurpitImportedDeviceOnboardView, ImportDevices
from .setting import SettingsView
from .datamapping import DataMappingView
from .reconcile import (
    ReconcileView, 
    ReconcileDetailView, 
    SlurpitInitIPAddressListView, 
    SlurpitPrefixListView, 
    SlurpitInterfaceListView, 
    SlurpitPrefixEditView, 
    SlurpitInterfaceEditView, 
    SlurpitIPAddressEditView, 
    SlurpitInterfaceBulkEditView, 
    SlurpitPrefixBulkEditView, 
    SlurpitIPAddressBulkEditView,
)



# Generated by Django 5.0.6 on 2024-08-30 01:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("slurpit_netbox", "0019_alter_slurpitvlan_name"),
    ]

    operations = [
        migrations.AlterField(
            model_name="slurpitinitipaddress",
            name="ignore_description",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitinitipaddress",
            name="ignore_role",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitinitipaddress",
            name="ignore_status",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitinitipaddress",
            name="ignore_tenant",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitinitipaddress",
            name="ignore_vrf",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitinterface",
            name="ignore_duplex",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitinterface",
            name="ignore_module",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitinterface",
            name="ignore_speed",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitinterface",
            name="ignore_type",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitprefix",
            name="ignore_description",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitprefix",
            name="ignore_role",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitprefix",
            name="ignore_site",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitprefix",
            name="ignore_status",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitprefix",
            name="ignore_tenant",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitprefix",
            name="ignore_vlan",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitprefix",
            name="ignore_vrf",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitvlan",
            name="ignore_description",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitvlan",
            name="ignore_group",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitvlan",
            name="ignore_role",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitvlan",
            name="ignore_site",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitvlan",
            name="ignore_status",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitvlan",
            name="ignore_tenant",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AlterField(
            model_name="slurpitvlan",
            name="ignore_vid",
            field=models.BooleanField(default=False, null=True),
        ),
    ]



from netbox.api.routers import NetBoxRouter
from . import views

router = NetBoxRouter()
router.APIRootView = views.SlurpitRootView
router.register("device", views.DeviceViewSet)
router.register("test", views.SlurpitTestAPIView, basename="test")
router.register("netbox-device", views.SlurpitDeviceView)
router.register("ipam", views.SlurpitIPAMView)
router.register("interface", views.SlurpitInterfaceView)
router.register("prefix", views.SlurpitPrefixView)
router.register("planning", views.SlurpitPlanningViewSet)
router.register("vlan", views.SlurpitVLANView)
router.register("site", views.SlurpitSiteView)
app_name = 'slurpit_netbox-api'
urlpatterns = router.urls



# Generated by Django 5.0.6 on 2024-08-16 02:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("slurpit_netbox", "0018_slurpitvlan_ignore_description_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="slurpitvlan",
            name="name",
            field=models.CharField(blank=True, default="", max_length=64, null=True),
        ),
    ]



from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
from django.urls import reverse
from django.core.exceptions import ValidationError
from django.core.validators import MaxValueValidator, MinValueValidator
from django.utils.translation import gettext_lazy as _
from django.db.models import Q

from netbox.models import OrganizationalModel, PrimaryModel
from ipam.constants import VLAN_VID_MIN, VLAN_VID_MAX
from ipam.choices import VLANStatusChoices
from ipam.querysets import VLANQuerySet, VLANGroupQuerySet
from dcim.models import Interface
from virtualization.models import VMInterface
from urllib.parse import urlencode

class SlurpitVLAN(PrimaryModel):
    """
    A VLAN is a distinct layer two forwarding domain identified by a 12-bit integer (1-4094). Each VLAN must be assigned
    to a Site, however VLAN IDs need not be unique within a Site. A VLAN may optionally be assigned to a VLANGroup,
    within which all VLAN IDs and names but be unique.

    Like Prefixes, each VLAN is assigned an operational status and optionally a user-defined Role. A VLAN can have zero
    or more Prefixes assigned to it.
    """
    site = models.ForeignKey(
        to='dcim.Site',
        on_delete=models.PROTECT,
        related_name='slurpit_vlans',
        blank=True,
        null=True,
        help_text=_("The specific site to which this VLAN is assigned (if any)")
    )
    group = models.ForeignKey(
        to='ipam.VLANGroup',
        on_delete=models.PROTECT,
        related_name='slurpit_vlans',
        blank=True,
        null=True,
        help_text=_("VLAN group (optional)")
    )
    group = models.CharField(
        verbose_name=_('group'),
        max_length=64,
        blank=True,
        null=True,
    )
    vid = models.PositiveSmallIntegerField(
        verbose_name=_('VLAN ID'),
        validators=(
            MinValueValidator(VLAN_VID_MIN),
            MaxValueValidator(VLAN_VID_MAX)
        ),
        help_text=_("Numeric VLAN ID (1-4094)")
    )
    name = models.CharField(
        verbose_name=_('name'),
        max_length=64,
        default = _(''),
        blank=True,
        null=True
    )
    tenant = models.ForeignKey(
        to='tenancy.Tenant',
        on_delete=models.PROTECT,
        related_name='slurpit_vlans',
        blank=True,
        null=True
    )
    status = models.CharField(
        verbose_name=_('status'),
        max_length=50,
        choices=VLANStatusChoices,
        default=VLANStatusChoices.STATUS_ACTIVE,
        help_text=_("Operational status of this VLAN")
    )
    role = models.ForeignKey(
        to='ipam.Role',
        on_delete=models.SET_NULL,
        related_name='slurpit_vlans',
        blank=True,
        null=True,
        help_text=_("The primary function of this VLAN")
    )

    enable_reconcile = models.BooleanField(
        default=False,
        verbose_name=_('enable reconcile'),
    )
    
    ignore_status = models.BooleanField(
        default=False,
        null=True,
        verbose_name=_('ignore status'),
    )
    ignore_site = models.BooleanField(
        default=False,
        null=True,
        verbose_name=_('ignore site'),
    )
    ignore_group = models.BooleanField(
        default=False,
        null=True,
        verbose_name=_('ignore group'),
    )
    ignore_vid = models.BooleanField(
        default=False,
        null=True,
        verbose_name=_('ignore vid'),
    )
    ignore_role = models.BooleanField(
        default=False,
        null=True,
        verbose_name=_('ignore role'),
    )
    ignore_tenant = models.BooleanField(
        default=False,
        null=True,
        verbose_name=_('ignore tenant'),
    )
    ignore_description = models.BooleanField(
        default=False,
        null=True,
        verbose_name=_('ignore description'),
    )
    objects = VLANQuerySet.as_manager()

    clone_fields = [
        'site', 'tenant', 'status', 'role', 'description',
    ]

    class Meta:
        ordering = ('site', 'group', 'vid', 'pk')  # (site, group, vid) may be non-unique
        constraints = (
            models.UniqueConstraint(
                fields=('group', 'vid'),
                name='%(app_label)s_%(class)s_unique_group_vid'
            ),
            models.UniqueConstraint(
                fields=('group', 'name'),
                name='%(app_label)s_%(class)s_unique_group_name'
            ),
        )
        verbose_name = _('VLAN')
        verbose_name_plural = _('VLANs')

    def __str__(self):
        return f'{self.name} ({self.vid})'

    def get_absolute_url(self):
        return reverse('ipam:vlan', args=[self.pk])

    def clean(self):
        super().clean()

    def get_status_color(self):
        return VLANStatusChoices.colors.get(self.status)
    
    def get_edit_url(self):
        query_params = {'tab': "vlan"}
        base_url = reverse("plugins:slurpit_netbox:reconcile_list")
        # Encode your query parameters and append them to the base URL
        url_with_querystring = f"{base_url}?{urlencode(query_params)}"

        base_url = reverse('plugins:slurpit_netbox:slurpitvlan_edit', args=[self.pk])
        query_params = {'return_url': url_with_querystring}
        url_with_querystring = f"{base_url}?{urlencode(query_params)}"

        return url_with_querystring





from dcim.models import DeviceRole, DeviceType, Manufacturer, Site, Location, Region, SiteGroup, Rack
from extras.choices import CustomFieldTypeChoices
from extras.models import CustomField, CustomFieldChoiceSet, ConfigTemplate
from extras.models.tags import Tag
from core.models import ObjectType
from django.db.models import Q, Transform, CharField, TextField

from .. import get_config
from .device import SlurpitImportedDevice, SlurpitStagedDevice
from .planning import SlurpitPlanning, SlurpitSnapshot
from .setting import SlurpitSetting
from .mapping import SlurpitMapping
from .ipam import SlurpitInitIPAddress
from .interface import SlurpitInterface
from .prefix import SlurpitPrefix
from .vlan import SlurpitVLAN
__all__ = [
    'SlurpitImportedDevice', 'SlurpitStagedDevice', 'SlurpitInitIPAddress'
    'post_migration', 'SlurpitSetting'
]

def ensure_slurpit_tags(*items):
    if (tags := getattr(ensure_slurpit_tags, 'cache', None)) is None:
        name = 'slurpit'
        tag, _ = Tag.objects.get_or_create(name=name, defaults={'slug':name, 'description':'Slurp\'it onboarded', 'color': 'F09640'})

        dcim_applicable_to = 'device', 'devicerole', 'devicetype', 'manufacturer', 'site'
        ipam_applicable_to = 'iprange', 'prefix'
        slurpit_netbox_applicable_to = 'slurpitinitipaddress', 'slurpitinterface', 'slurpitprefix'

        dcim_Q = Q(app_label='dcim', model__in=dcim_applicable_to)
        ipam_Q = Q(app_label='ipam', model__in=ipam_applicable_to)
        slurpit_Q = Q(app_label='slurpit_netbox', model__in=slurpit_netbox_applicable_to)

        tagged_types = ObjectType.objects.filter(ipam_Q | dcim_Q | slurpit_Q)
        tag.object_types.set(tagged_types.all())
        tags = {tag}
        ensure_slurpit_tags.cache = tags
    for item in items:
        item.tags.set(tags)
    return tags

def create_custom_fields():   
    device = ObjectType.objects.get(app_label='dcim', model='device')
    cf, _ = CustomField.objects.get_or_create(
                name='slurpit_hostname',   
                defaults={            
                    "type":CustomFieldTypeChoices.TYPE_TEXT,
                    "description":"",
                    "is_cloneable":True,
                    "label":'Hostname',
                    "group_name":"Slurp'it"
                })
    cf.object_types.set({device})

    cf, _ = CustomField.objects.get_or_create(
                name='slurpit_fqdn',  
                defaults={                     
                    "type":CustomFieldTypeChoices.TYPE_TEXT,
                    "description":"",
                    "is_cloneable":True,
                    "label":'Fqdn',
                    "group_name":"Slurp'it"
                })
    cf.object_types.set({device})
        
    cf, _ = CustomField.objects.get_or_create(
                name='slurpit_platform',
                defaults={            
                    "type":CustomFieldTypeChoices.TYPE_TEXT,
                    "description":"",
                    "is_cloneable":True,
                    "label":'Platform',
                    "group_name":"Slurp'it",
                })
    cf.object_types.set({device})

    cf, _ = CustomField.objects.get_or_create(
                name='slurpit_manufacturer', 
                defaults={            
                    "type":CustomFieldTypeChoices.TYPE_TEXT,
                    "description":"",
                    "is_cloneable":True,
                    "label":'Manufacturer',
                    "group_name":"Slurp'it",
                })
    cf.object_types.set({device})
    
    cf, _ = CustomField.objects.get_or_create(
                name='slurpit_devicetype',
                defaults={            
                    "type":CustomFieldTypeChoices.TYPE_TEXT,
                    "description":"",
                    "is_cloneable":True,
                    "label":'Device Type',
                    "group_name":"Slurp'it",
                })
    cf.object_types.set({device})
    
    cf, _ = CustomField.objects.get_or_create(
                name='slurpit_ipv4',
                defaults={            
                    "type":CustomFieldTypeChoices.TYPE_TEXT,
                    "description":"",
                    "is_cloneable":True,
                    "label":'Ipv4',
                    "group_name":"Slurp'it",
                })
    cf.object_types.set({device})

    cf, _ = CustomField.objects.get_or_create(
                name='slurpit_site',
                defaults={            
                    "type":CustomFieldTypeChoices.TYPE_TEXT,
                    "description":"",
                    "is_cloneable":True,
                    "label":'Site',
                    "group_name":"Slurp'it",
                })
    cf.object_types.set({device})

def create_default_data_mapping():
    SlurpitMapping.objects.all().delete()
    
    mappings = [
        {"source_field": "hostname", "target_field": "device|name"},
        {"source_field": "fqdn", "target_field": "device|primary_ip4"},
        {"source_field": "ipv4", "target_field": "device|primary_ip4"},
        {"source_field": "device_os", "target_field": "device|platform"},
        {"source_field": "device_type", "target_field": "device|device_type"},
    ]
    for mapping in mappings:
        SlurpitMapping.objects.get_or_create(**mapping)

def add_default_mandatory_objects(tags):
    site, _ = Site.objects.get_or_create(**get_config('Site'))
    site.tags.set(tags)

    manu, _ = Manufacturer.objects.get_or_create(**get_config('Manufacturer'))
    manu.tags.set(tags)

    dtype, _ = DeviceType.objects.get_or_create(manufacturer=manu, **get_config('DeviceType'))
    dtype.tags.set(tags)

    role, _ = DeviceRole.objects.get_or_create(**get_config('DeviceRole'))
    role.tags.set(tags)    

    create_default_data_mapping()


def post_migration(sender, **kwargs):
    create_custom_fields()
    tags = ensure_slurpit_tags()
    add_default_mandatory_objects(tags)

class LowerCase(Transform):
    lookup_name = "lower"
    function = "LOWER"

CharField.register_lookup(LowerCase)
TextField.register_lookup(LowerCase)


from netbox.views import generic
from netbox.api.viewsets import NetBoxModelViewSet
from utilities.exceptions import AbortRequest, PermissionsViolation
from utilities.forms import restrict_form_fields
from dcim.choices import DeviceStatusChoices
from dcim.models import  Manufacturer, Platform, DeviceType, Site, SiteGroup, Location, Device, DeviceRole, Region
from dcim.api.serializers import DeviceSerializer
from dcim.filtersets import DeviceFilterSet
from ipam.models import *
from extras.models import CustomField


# Generated by Django 4.2.9 on 2024-01-29 08:16

from django.db import migrations, models
import taggit.managers
import utilities.json


class Migration(migrations.Migration):

    dependencies = [
        ('extras', '0105_customfield_min_max_values'),
        ('slurpit_netbox', '0003_alter_slurpitimporteddevice_ipv4_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='SlurpitMapping',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('source_field', models.CharField(max_length=255, unique=True)),
                ('target_field', models.CharField(max_length=255)),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.db import models
from django.utils.translation import gettext as _
from netbox.models import PrimaryModel
from ..management.choices  import SlurpitApplianceTypeChoices


class SlurpitSetting(PrimaryModel):
    server_url = models.CharField(max_length=200, verbose_name=_("URL"))
    api_key = models.CharField(max_length=50,editable=False)
    last_synced = models.DateTimeField(blank=True, auto_now=True,null=True, editable=False)
    connection_status = models.CharField(max_length=50,editable=False, null=True, default='')
    push_api_key = models.CharField(max_length=200,null=True, editable=False)
    
    appliance_type = models.CharField(
        verbose_name=_('Applicance Type'),
        max_length=50,
        choices=SlurpitApplianceTypeChoices,
        blank=True
    )

    class Meta:
        verbose_name = "setting"
        verbose_name_plural = "setting"

    def __str__(self):
        return f"{self.server_url}"

    def get_absolute_url(self):
        return '/'

    @property
    def docs_url(self):
        # TODO: Add docs url
        return ""

    def clean(self):
        super().clean()
        


# Generated by Django 5.0.9 on 2024-10-10 09:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "slurpit_netbox",
            "0020_alter_slurpitinitipaddress_ignore_description_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="slurpitimporteddevice",
            name="site",
            field=models.CharField(max_length=255, null=True),
        ),
        migrations.AddField(
            model_name="slurpitstageddevice",
            name="site",
            field=models.CharField(max_length=255, null=True),
        ),
    ]
